module Bindings (display, reshape, keyboardMouse, looper) where

import Data.IORef
import Display
import Engine
import Graphics.UI.GLUT
import System.Exit

-- Just re-render
reshape :: ReshapeCallback
reshape s = do
  clear [ColorBuffer]
  postRedisplay Nothing

keyboardMouse ::
  IORef Cells ->
  IORef GLfloat ->
  IORef (GLfloat, GLfloat) ->
  IORef Int ->
  IORef Bool ->
  KeyboardMouseCallback
keyboardMouse cells z p t s key Down _ position@(Position x y) = do
  case key of
    (SpecialKey KeyF11) -> fullScreenToggle
    (Char 'q') -> do exit; exitSuccess
    (Char 'r') -> cells $~! const emptymem
    (Char 'z') -> t $~! \x -> x + 50
    (Char 'x') -> t $~! \x -> if x > 50 then x -50 else 50
    (Char 'f') -> get s >>= \x -> if x then cells $~! updatem else pure ()
    (Char ' ') -> s $~! not -- Forward one step
    (Char 'a') -> do
      k <- get z
      if k > 1
        then pure ()
        else
          z $~! const (k * 1.5)
            >> p $~! \(x, y) -> (x * 1.5, y * 1.5)
    (Char 's') ->
      (z $~! \x -> x / 1.5)
        >> p $~! \(x, y) -> (x / 1.5, y / 1.5)
    (SpecialKey KeyLeft) -> p $~! \(x, y) -> (x + 0.1, y)
    (SpecialKey KeyRight) -> p $~! \(x, y) -> (x -0.1, y)
    (SpecialKey KeyUp) -> p $~! \(x, y) -> (x, y -0.1)
    (SpecialKey KeyDown) -> p $~! \(x, y) -> (x, y + 0.1)
    (Char 'h') -> p $~! \(x, y) -> (x + 0.1, y)
    (Char 'l') -> p $~! \(x, y) -> (x -0.1, y)
    (Char 'j') -> p $~! \(x, y) -> (x, y -0.1)
    (Char 'k') -> p $~! \(x, y) -> (x, y + 0.1)
    (MouseButton LeftButton) -> do
      Size w' h' <- get windowSize
      zoom <- get z
      c <- get cells
      (posx, posy) <- get p
      let w = fromIntegral w'
      let h = fromIntegral h'
      let k = zoom * fromIntegral h' / 20

      let (p, q) =
            ( floor $ (fromIntegral (x - w `div` 2) + k / 2) / k - posx * 10 / zoom,
              floor $ - (fromIntegral (y - h `div` 2) - k / 2) / k - posy * 10 / zoom
            )

      cells $~! const (toggle c (fromIntegral p, fromIntegral q))
    _ -> return ()

  postRedisplay Nothing
keyboardMouse _ _ _ _ _ _ _ _ _ = return ()

looper :: IORef Cells -> IORef Int -> IORef Bool -> TimerCallback
looper cells n s = do
  time <- get n
  stop <- get s
  old <- get cells
  if stop
    then pure ()
    else do
      cells $~! const (updatem old)
      postRedisplay Nothing
  addTimerCallback time (looper cells n s)
