{-# LANGUAGE BangPatterns #-}
module Engine (startingmem, Cells, updatem, toggle, emptymem, pointlist) where

-- Naive Method
import qualified Data.Set as S

data Cells = Cells {alive :: S.Set (Integer, Integer), changed :: S.Set (Integer, Integer)}
    deriving (Show)


pointlist :: Cells -> [(Integer, Integer)]
pointlist = S.toList . alive

emptymem :: Cells
emptymem = Cells S.empty S.empty 

startingmem :: [(Integer, Integer)] -> Cells
startingmem x = Cells (S.fromList x) (S.fromList x)

--Returns moore neighborhood of given cell without the central cell
neighbors :: (Num a, Num b, Enum a, Enum b, Eq a, Eq b) => (a, b) -> [(a, b)]
neighbors (x, y) = [(x+k1, y+k2) | 
                    k1 <- [-1..1],
                    k2 <-[-1..1], 
                    not $ k1 == 0 && k2 == 0]

--Checks how many neighbors are alive
check :: (Ord a, Ord b, Num a, Num b, Enum a, Enum b) => S.Set (a, b) -> (a, b) -> Int
check !mem (x, y) = length . filter id . fmap (`S.member` mem) $ neighbors (x, y)

-- toggles a single cell
toggle :: Cells -> (Integer, Integer) -> Cells
toggle !mem (x, y) = if S.member (x, y) a then Cells (S.delete (x, y) a) tmp else Cells (S.insert (x, y) a) tmp
    where a = alive mem 
          c = changed mem
          tmp = S.insert (x, y) (S.insert (x-1, y) c) -- We add a nearby cell so that (x, y) will be updated as neighbour of a changed cell.



updatem :: Cells -> Cells
updatem !mem = Cells (S.union toKeep toAdd) (S.union toAdd toRemove) where
          c = changed mem
          a = alive mem
          frontier = S.partition (`S.member` a) . S.unions . S.map (S.fromList . neighbors) $ c
          toAdd = S.filter born (snd frontier)
          toRemove = S.filter (not . live) (fst frontier)
          toKeep = S.difference a toRemove
          live x = 2 == check a x || 3 == check a x -- This part can be abstracted to easily support more automatas
          born x = 3 == check a x